import os, subprocess

'''A build system has three methods: "configure" (with optionak "force" parameter), "build" and "install"'''

# Compile, build, and install cmake projects:
class CMake:
	def __init__(self, sourceFolder, buildFolder, module, config):
		self.sourceFolder = os.path.abspath(sourceFolder)
		self.buildFolder = os.path.abspath(buildFolder)
		self.installDir = config['installDir']
		self.buildType = config['buildType']
		self.jobs = config['jobs']
		self.buildCmdPrefix = config['buildCmdPrefix']
		self.installCmdPrefix = config['installCmdPrefix']
		self.cmakeParameters = module.get('cmakeParameters', [])
	
	def configure(self, force=False):
		if not os.path.exists(self.buildFolder): os.makedirs(self.buildFolder)
		os.chdir(self.buildFolder)
		# check if we actually need to work
		cacheFile = 'CMakeCache.txt'
		if os.path.exists(cacheFile) and os.path.exists('Makefile') and not force: return
		# yes we do! make sure we start clean, and then go ahead
		if os.path.exists(cacheFile): os.remove(cacheFile)
		os.putenv('PKG_CONFIG_PATH', os.path.join(self.installDir, 'lib', 'pkgconfig')) # I found no way to do this within cmake
		subprocess.check_call(['cmake', self.sourceFolder, '-DCMAKE_BUILD_TYPE='+self.buildType,
			'-DCMAKE_INSTALL_PREFIX='+self.installDir]+self.cmakeParameters)
		os.unsetenv('PKG_CONFIG_PATH')
	
	def build(self):
		os.chdir(self.buildFolder)
		subprocess.check_call(self.buildCmdPrefix + ['make', '-j'+str(self.jobs)])
	
	def install(self):
		os.chdir(self.buildFolder)
		subprocess.check_call(self.installCmdPrefix + ['make', 'install'])

# if auto-debuild is available, provide a wrapper for it
try:
	import auto_debuild
	class AutoDebuild:
		def __init__(self, sourceFolder, buildFolder, module, vcs, config):
			self.sourceFolder = os.path.abspath(sourceFolder)
			buildSystem = module.get('build-system', 'cmake')
			self.autoDebuildConfig = {
				'sourceName': module['name'],
				'debDir': os.path.abspath(config['debDir']),
				'buildSystem': buildSystem,
				'buildDir': os.path.abspath(buildFolder),
				'name': config['debName'],
				'email': config['debEMail'],
			}
			self.copyFromModule(module, 'buildDepends')
			if buildSystem == 'cmakeParameters':
				self.copyFromModule(module, 'cmakeParameters')
			self.vcs = vcs
			self.configured = False # make sure configure is called before build/install

		def copyFromModule(self, module, name):
			if name in module:
				self.autoDebuildConfig[name] = module[name]

		def configure(self, force=False): # force is ignored
			self.autoDebuildConfig['version'] = self.vcs.version() # by now, data has been fetched, so this is possible
			self.vcs.ignore('/debian/') # make sure the debian folder is ignored
			os.chdir(self.sourceFolder)
			print self.autoDebuildConfig
			self.files = auto_debuild.createDebianFiles(self.autoDebuildConfig)
			self.configured = True

		def build(self):
			if not self.configured: self.configure()
			os.chdir(self.sourceFolder)
			auto_debuild.buildDebianPackage(self.autoDebuildConfig)

		def install(self):
			if not self.configured: self.configure()
			os.chdir(self.sourceFolder)
			subprocess.check_call(['sudo', 'dpkg', '--install'] + self.files)
except ImportError:
	print "auto_debuild not found, disabling auto-debuild system"
	pass
