import os, git, subprocess

'''A VCS must have an "update" method with an optional "force" parameter.'''

# Fetch updates from git
class Git:
	def __init__(self, folder, url, commit):
		self.folder = os.path.abspath(folder)
		self.url = url
		self.commit = commit

	class _ProgressPrinter(git.remote.RemoteProgress):
		def update(self, op_code, cur_count, max_count=None, message=''):
			print self._cur_line+(" "*30)+"\r",

	def update(self):
		isBranch = (self.commit.startswith('origin/'))
		if isBranch:
			branchname = self.commit[len('origin/'):]
		else:
			branchname = "tag"
		# get us a git repository, and the "origin" remote
		if os.path.exists(self.folder):
			# load existing repo
			repo = git.Repo(self.folder)
			origin = repo.remotes.origin
		else:
			# create a new one
			os.makedirs(self.folder)
			repo = git.Repo.init(self.folder)
			origin = repo.create_remote('origin', self.url)
		origin.fetch(progress=Git._ProgressPrinter()) # download new data
		print " "*80+"\r", # clean the line we are in
		# create/find correct branch
		if branchname in repo.heads:
			branch = repo.heads[branchname]
		else:
			branch = repo.create_head(branchname, self.commit)
			if isBranch:
				branch.set_tracking_branch(origin.refs[branchname])
		# update it to the latest remote commit
		branch.checkout()
		repo.git.rebase(self.commit)
		print "...done",
		if repo.head.reference.commit != repo.refs[self.commit].commit:
			print "(keeping local patches around)",
		print

	def version(self):
		repo = git.Repo(self.folder)
		v = repo.git.describe()
		if v.startswith('v'): v = v[1:]
		return v

	def ignore(self, name):
		with open(os.path.join(self.folder, '.git', 'info', 'exclude'), 'r+') as f:
			for line in f:
				line = line.replace('\n', '').replace('\r', '')
				if line == name: return # is already ignored
			print >>f, name # add ignore line

class KDEGit(Git):
	def __init__(self, folder, name, commit):
		Git.__init__(self, folder, 'kde:'+name, commit)

# Fetch updates via SVN
class SVN:
	def __init__(self, folder, svnPath):
		self.folder = os.path.abspath(folder)
		self.svnPath = svnPath
	
	def update(self):
		if os.path.exists(self.folder):
			os.chdir(self.folder) # go into repository
			subprocess.check_call(['svn', 'switch', self.svnPath]) # and update to the URL we got
		else:
			subprocess.check_call(['svn', 'co', self.svnPath, self.folder])# just download it

class KDESVN(SVN):
	def __init__(self, folder, svnPath):
		SVN.__init__(self, folder, 'svn://svn.kde.org/home/kde/'+svnPath)
